package com.yixin.ms.server;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.excel.util.CollectionUtils;
import com.sun.org.apache.xpath.internal.operations.Bool;
import com.yixin.ms.boot.uitls.Preconditions;
import com.yixin.ms.boot.uitls.RedisUtil;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.enums.*;
import com.yixin.ms.model.vo.HealtCodeNucleinVO;
import com.yixin.ms.model.vo.HealtCodeTypeVO;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.server.dto.ScenicVerifyDTO;
import com.yixin.ms.server.vo.HealthOpenReturnVO;
import com.yixin.ms.server.vo.PlaceInfoVO;
import com.yixin.ms.server.vo.ScenicVerifyVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * @description:
 * @ClassName HealthOpenReturnServer
 * @Author WXG
 * @Date 2022/10/20 17:34
 */
@Slf4j
@Service
public class HealthOpenReturnServer {

    @Autowired
    private ScenicServer scenicServer;

    @Autowired
    private RedisUtil redisUtil;

    /**
     * 65周岁预约不限制通行
     */
    private static final int AGE = 65;

    public HealthOpenReturnVO healthOpenReturn(HealtCodeUserVO userVO, PlaceInfoVO placeInfoVO, ShortTermDTO shortTermDTO) {

        //目前只有衢通码在用
        userVO.setShowQrcode(placeInfoVO.getShowQrcode());
        userVO.setQrcodeText(placeInfoVO.getShowQrcode() ? placeInfoVO.getQrcodeText() : StrUtil.EMPTY);
        userVO.setAdvertise(placeInfoVO.getAdvertise());

        HealthOpenReturnVO healthOpenReturnVO = new HealthOpenReturnVO();
        /*用户健康码信息*/
        HealtCodeTypeVO codeTypeVO = userVO.getData().stream().findFirst().get();
        //用户核酸信息
        List<HealtCodeNucleinVO> nucleinList = userVO.getNuclein();
        Integer nucleicExpirationTime = placeInfoVO.getNucleicExpirationTime();
        Boolean noXcOpenDoor = placeInfoVO.getNoXcOpenDoor();
        Boolean noHsOpenDoor = placeInfoVO.getNoHsOpenDoor();
        Boolean riskXcOpenDoor = placeInfoVO.getRiskXcOpenDoor();
        String noAllowAreaCode = placeInfoVO.getNoAllowAreaCode();
        Boolean xcNeedCheckOpenDoor = placeInfoVO.getXcNeedCheckOpenDoor();
        Boolean isThroughBlackCheck = placeInfoVO.getIsThroughBlackCheck();
        String notAllowAge = placeInfoVO.getNotAllowAge();
        String notAllowAgeMsg = placeInfoVO.getNotAllowAgeMsg();
        Boolean dbIsAlarm = placeInfoVO.getIsAlarm();
        Integer placeType = placeInfoVO.getPlaceType();

        Boolean limitGather = Preconditions.isBlank(placeInfoVO.getLimitGather()) ? Boolean.FALSE : (placeInfoVO.getLimitGather().intValue() == 0 ? Boolean.FALSE : Boolean.TRUE);
        boolean nucleicExpirationFlag = false;
        Boolean isNoAllowAreaCode = true;//是否是允许通行的身份证地区
        /*
         * 身份证判定校验
         * */
        if (StrUtil.isNotEmpty(noAllowAreaCode) && StrUtil.isNotEmpty(userVO.getIdNumber())) {
            String[] split = noAllowAreaCode.split("/");
            for (String areaCode : split) {
                if (userVO.getIdNumber().startsWith(areaCode)) {
                    isNoAllowAreaCode = false;
                    break;
                }
            }
        }
        String openMsg = "";
        String openVoiceMsg = "";
        /**
         * 开门规则判定校验
         */
        if (placeInfoVO != null && placeInfoVO.getScenicFlag()) {
            userVO.setIsOpen(HealthOpenEnum.RELEASE.getCode());
            openMsg = "欢迎入园";
            openVoiceMsg = openMsg;
        } else {
            userVO.setIsOpen(HealthOpenEnum.RELEASE.getCode());
            openMsg = "请通行";
            openVoiceMsg = openMsg;
        }

        // 景区场景，需要判断有无预约
        if (Preconditions.isNotBlank(placeInfoVO)
                && placeInfoVO.getScenicFlag()
                && Preconditions.isNotBlank(userVO.getIdNumber())
                && IdcardUtil.getAgeByIdCard(userVO.getIdNumber()) < AGE) {
            //判断是否小于65周岁查询预约放行权限
            ScenicVerifyVO verify = scenicServer.verify(new ScenicVerifyDTO().setIdNumber(userVO.getIdNumber()), placeInfoVO.getAdminId());
            if (Preconditions.isNotBlank(verify) && !verify.getReleaseStatus()) {
//                未预约 请您先预约
                userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                openMsg = verify.getMessage();
                openVoiceMsg = openMsg;
            }
        }

        //黑名单校验
        if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0) && !isThroughBlackCheck) {
            userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
            openMsg = "未授权，请联系管理员";
            openVoiceMsg = openMsg;
        }

        //不被允许通行的身份证地区
        if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0) && !isNoAllowAreaCode) {
            userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
            openMsg = "禁止通行";
            openVoiceMsg = openMsg;
        }

        //TODO。。。身份证校验是否未满18周岁 目前未加逻辑 默认false 通行区间
        if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0)
                && Preconditions.isNotBlank(userVO.getIdNumber())
                && Preconditions.isNotBlank(notAllowAge)
                && Preconditions.isNotBlank(placeType)
                && (placeType.intValue() == 0 || placeType.intValue() == 1)) {
            int age = IdcardUtil.getAgeByIdCard(userVO.getIdNumber());
            String[] list = notAllowAge.split("-");
            if (list.length == 2
                    && (age >= Integer.parseInt(list[0]) && age <= Integer.parseInt(list[1]))) {
                userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                openMsg = "禁止通行";
                if (Preconditions.isNotBlank(notAllowAgeMsg)) {
                    openMsg = notAllowAgeMsg;
                }
                openVoiceMsg = openMsg;
            }
        }

        if (Preconditions.isNotBlank(placeInfoVO)
                && Preconditions.isNotBlank(placeInfoVO.getIsOpenHealthCode())
                && placeInfoVO.getIsOpenHealthCode().intValue() == 1) {//开启健康码

            //红黄码
            if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0) && !HealthCodeLevelEnum.GREEN.getDesc().equals(codeTypeVO.getMzt())) {
                userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                openMsg = codeTypeVO.getMzt() + "禁止通行";
                openVoiceMsg = openMsg;
                if (HealthCodeLevelEnum.RED.getDesc().equals(codeTypeVO.getMzt())) {
                    codeTypeVO.setContent(codeTypeVO.getMzt() + "：健康码状态为高风险").setContentColor(ColorEnum.RED.getValue());
                    userVO.setBackgroundColor(ColorEnum.RED.getValue());
                } else {
                    codeTypeVO.setContent(codeTypeVO.getMzt() + ": 健康码状态为中风险").setContentColor(ColorEnum.YELLOW.getValue());
                    userVO.setBackgroundColor(ColorEnum.YELLOW.getValue());
                }
            }

            //三天三检校验
            if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0) && limitGather
                    && (Preconditions.isBlank(userVO.getLimitGather()) || userVO.getLimitGather())) {
                userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                openMsg = "未完成三天三检，禁止通行";
                openVoiceMsg = openMsg;
            }

            //去过中高风险地区
            if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0) && !riskXcOpenDoor && StrUtil.isNotEmpty(userVO.getTravelValidation()) && TravelValidationEnum.HAVE_BEEN_TO.getCode().equals(userVO.getTravelValidation())) {
                userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                openMsg = TravelValidationEnum.HAVE_BEEN_TO.getDesc() + ",禁止通行";
                openVoiceMsg = openMsg;
            }

            //核酸校验
            if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0) && noHsOpenDoor && nucleicExpirationTime != null && !nucleicExpirationTime.equals(0)) {
                if ((CollectionUtils.isEmpty(nucleinList) || DateUtil.betweenMs(nucleinList.stream().findFirst().get().getShowTime(), DateUtil.date()) > nucleicExpirationTime * 60 * 60 * 1000)) {
                    nucleicExpirationFlag = true;
                    userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                    if (nucleicExpirationTime <= NucleicRuleEnum.SEVENTY_TWO.getValue()) {
                        openMsg = new StringBuilder().append(nucleicExpirationTime).append("小时内无有效核酸阴性记录，禁止通行").toString();
                    } else {
                        openMsg = new StringBuilder().append((nucleicExpirationTime / 24) + 1).append("天内无有效核酸阴性记录，禁止通行").toString();
                    }
                    openVoiceMsg = openMsg;
                }
            }

            if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0) && !noHsOpenDoor && nucleicExpirationTime != null && !nucleicExpirationTime.equals(0) && (CollectionUtils.isEmpty(nucleinList) || nucleinList.stream().findFirst().get().getReportTime() == null || DateUtil.betweenMs(nucleinList.stream().findFirst().get().getReportTime(), DateUtil.date()) > nucleicExpirationTime * 60 * 60 * 1000)) {
                userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                if (CollectionUtil.isNotEmpty(nucleinList) && DateUtil.betweenMs(nucleinList.stream().findFirst().get().getChecktime(), DateUtil.date()) < 24 * 60 * 60 * 1000) {
                    openMsg = "24小时核酸结果未出，禁止通行";
                } else {
                    openMsg = nucleicExpirationTime + "小时内无有效核酸阴性记录，禁止通行";
                }
                openVoiceMsg = openMsg;
            }
            //配置了没有行程不开门的账号
            if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0) && !noXcOpenDoor && (StrUtil.isEmpty(userVO.getTravelValidation()) || TravelValidationEnum.NO_ITINERARY.getCode().equals(userVO.getTravelValidation()))) {
                userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                openMsg = TravelValidationEnum.NO_ITINERARY.getDesc() + ",需人工核验";
                openVoiceMsg = openMsg;
            }


            //行程码需检测不开门
            if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0) && !xcNeedCheckOpenDoor && StrUtil.isNotEmpty(userVO.getTravelValidation()) && TravelValidationEnum.NEEDCHECK.getCode().equals(userVO.getTravelValidation())) {
                userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                openMsg = "禁止通行,需检测";
                openVoiceMsg = openMsg;
            }

            //上城分局定制化需求
            if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0) && redisUtil.hasKey("hh_no_nucleic_report")) {
                String supremeAdminIdStr = (String) redisUtil.get("hh_no_nucleic_report");
                if (Preconditions.isNotBlank(supremeAdminIdStr)) {
                    List<String> adminIds = Arrays.asList(supremeAdminIdStr.split("::"));
                    if (adminIds.contains(String.valueOf(placeInfoVO.getAdminId()))) {
                        if (Preconditions.isNotBlank(nucleinList)) {
                            HealtCodeNucleinVO nucleinVO = nucleinList.stream().findFirst().get();
                            if (Preconditions.isBlank(nucleinVO)
                                    || (Preconditions.isNotBlank(nucleinVO.getReportTime())
                                    && Preconditions.isNotBlank(nucleinVO.getChecktime())
                                    && DateUtil.compare(nucleinVO.getChecktime(), nucleinVO.getReportTime()) > 0)
                                    && System.currentTimeMillis() - nucleinVO.getChecktime().getTime() > 43200000) {
                                userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                                openMsg = "12小时内结果未出，禁止通行";
                                openVoiceMsg = openMsg;
                            }
                        } else {
                            userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                            openMsg = "未查询到核酸结果，禁止通行";
                            openVoiceMsg = openMsg;
                        }
                    }
                }
            }
        }

        if (userVO.getIsOpen() != null
                && userVO.getIsOpen().intValue() == HealthOpenEnum.NO_THOROUGHFARE.getCode().intValue()) {
            if (dbIsAlarm) {
                userVO.setIsAlarm(Boolean.TRUE);
            }
            userVO.setVaccineIconUrl("https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/warning.png");
            userVO.setVaccineNumColor("#ed593b");
        }
        userVO.setOpenMsg(openMsg);
        userVO.setOpenVoiceMsg(openVoiceMsg);
        if (Preconditions.isNotBlank(codeTypeVO)) {
            codeTypeVO.setContent(openMsg);
            userVO.setData(Arrays.asList(codeTypeVO));
        }
        /*开门规则校验--结束*/
        healthOpenReturnVO.setNucleicExpirationFlag(nucleicExpirationFlag);
        return healthOpenReturnVO;
    }
}
